const path = require('path');
const projectRoot = process.cwd();

const build = require('./build');
const FileListPlugin = require('./build/FileListPlugin');
const chalk = require('chalk');

// 获取命令中的dist参数
const argv = process.argv;
let distPath = path.join(projectRoot, `./dist`);
argv.map(item => {
  if (item.indexOf('dist=') >= 0) {
    distPath = path.join(projectRoot, item.replace('dist=', ''));
  }
});
const publicPath = `/${build.remoteAppName}/`;
module.exports = {
  layoutRoot: path.join(__dirname, 'src'),
  templatePath: path.join(__dirname, 'src/index.ejs'),
  publicPath, // 子应用base路径， 默认为包名
  outputPath: distPath, // 打包文件输出地址；配置值时，基于项目根目录，默认为：'./libDist'
  isBundleAnalyzerPlugin: false,
  packEntry: {},
  // lessVariables: {
  //   'primary-color': '#ff0000',
  //   'primaryColor': '#ff0000',
  //   '@primary-color': '#000'
  // },
  webpackConfig: (config, options) => {
    config.resolve.alias = {
      'Component': path.join(__dirname, './src/component')
    };

    config.devServer.proxy = {
      '/web': {
        target: 'http://www.icoderun.com',
        changeOrigin: true,
      },
      // '/lowcodeApi': {
      //   target: 'http://www.icoderun.com',
      //   changeOrigin: true,
      // },
      '/staticImages': {
        target: 'http://www.icoderun.com',
        changeOrigin: true,
      }
    };

    config.optimization.splitChunks = {
      chunks: 'async',
      minSize: 50000,
      maxSize: 1000000,
      minChunks: 1,
      maxAsyncRequests: 6,
      maxInitialRequests: 1,
      automaticNameDelimiter: '~',
      cacheGroups: {
        echarts: {
          name: `chunk-echarts`,
          test: /[\\/]echarts[\\/]/,
          priority: -5,
          chunks: 'all',
          enforce: true,
          reuseExistingChunk: true,
        },
        vendors: {
          name: `chunk-vendors`,
          test: /[\\/]node_modules[\\/]/,
          priority: -10,
          chunks: 'initial',
          reuseExistingChunk: true,
        },
        common: {
          name: `chunk-common`,
          minChunks: 2,
          priority: -20,
          chunks: 'initial',
          reuseExistingChunk: true,
        },
      },
    }

    let propsConfigsString = '';
    try{
      propsConfigsString = JSON.stringify(build.propsConfigs);
    }
    catch(e) {
      console.log(chalk.red('Error for propsConfigs:'), e);
    }

    let listString = '';
    try{
      listString = JSON.stringify(build.list);
    }
    catch(e) {
      console.log(chalk.red('Error for list:'), e);
    }
    console.log('组件:', build.remoteAppName, build.exposesComponents);
    config.plugins.push(
      new options.moduleFederationPlugin({
        name: build.remoteAppName,
        filename: 'remoteEntry.js',
        library: { type: 'var', name: build.remoteAppName },
        exposes: {
          ...build.exposesComponents,
        },
        shared: {
          react: { singleton: true },
          'react-dom': { singleton: true },
        },
      }),

      new options.monacoWebpackPlugin({
        languages: ['javascript', 'css', 'json']
      }),

      

      new FileListPlugin([
        {
          content: `export default ${propsConfigsString}`, //propsOptionsContent,
          fileName: 'data/propsConfig.js',
        },
      ]),

      new FileListPlugin([
        {
          content: listString, //propsOptionsContent,
          fileName: 'data/list.json',
        },
      ]),

      new options.offlinePlugin({
        appShell: publicPath,
        excludes: ['.htaccess', '**/*.html'],
        caches: {
          main: [':rest:'],
          additional: ['*.chunk.js'],
        },
        safeToUseOptionalCaches: true,
        ServiceWorker: {
          publicPath: publicPath+'sw.js?t=' + +new Date()
        }
      })
    );

    return config;
  },
};
